
%% LOWW (2005) Benchmark model
%% Compare Dynare solution to Sims' solution in R
%% Uses different mode as LOWW

%----------------------------------------------------------------
% 0. Housekeeping (close all graphic windows)
%----------------------------------------------------------------

close all;

%----------------------------------------------------------------
% 1. Defining variables
%----------------------------------------------------------------

%% Declare Endogenous Vars
var c r pie q inv k u r_k l w y eps_b eps_i eps_g eps_a eps_l pi_bar c_star inv_star y_star w_star l_star q_star r_star r_k_star k_star u_star;

%% Declare Exogenous Vars
varexo eta_w eta_pie eta_q eta_r nu_b nu_i nu_g nu_a nu_l nu_pi_bar;

%% Declare Parameters
parameters thet bet sig delt zet R_bar_k psi c_y g_y k_y phi alph gam_w lamb_w xi_w chi gam_p xi_p rho_b rho_i rho_g rho_a rho_l rho_pi_bar sig_b sig_i sig_g sig_a sig_l sig_pie sig_pie_bar sig_q sig_r sig_w r_i r_pi r_y r_delta_pi r_delta_y AA BB CC DD EE FF log_psi_inv phi_1;

%----------------------------------------------------------------
% 2. Calibration
%----------------------------------------------------------------

%% Initialize hyper-parameters
zet= 0.20241;
log_psi_inv= 2.42;
phi_1= 0.1071;
chi= 0.75265;
xi_p= 0.7974;
xi_w= 0.2705 ;
gam_w= 0.4492 ; 
gam_p= 0.1064  ; 
thet= 0.3698;
sig= 2.1673;
rho_b= 0.9054;
rho_i= 0.2516 ;
rho_g= 0.9729;
rho_a= 0.9462 ; 
rho_l= 0.9815;
rho_pi_bar= 0.99;
sig_b= 2.5144;
sig_i= 4.4334;
sig_g= 2.00;
sig_a= 0.5906;
sig_l= 2.6591;
sig_pie_bar= 0.08169;
sig_q= 3.555;
sig_pie= 4.074 ; 
sig_r= 0.3397;
sig_w= 0.3021;
r_i= 0.6433;
r_pi= 2.0674;
r_y= 0.10857;
r_delta_pi= 0.2287;
r_delta_y= 0.4566;

% Calibrated params
delt=0.025; 
R_bar_k=0.0351;  
c_y=0.56; 
g_y=0.2; 
k_y=9.6; 
alph=0.36;
lamb_w=0.2; 

%% other derived parameters:
bet=1/(1-delt+R_bar_k);
AA=1/(1+thet+bet*thet^2);
BB=1+bet*thet+bet*thet^2;
CC=1/(1-delt+R_bar_k);
DD=(lamb_w*(1-bet*xi_w)*(1-xi_w))/((lamb_w+(1+lamb_w)*chi)*xi_w);
EE=sig/((1-thet)*(1-bet*thet));
FF=(1-bet*xi_p)*(1-xi_p)/xi_p;

%----------------------------------------------------------------
% 3. Model
%----------------------------------------------------------------

%% Model Declaration
model(linear);
#phi=1+phi_1;
#psi=1/exp(log_psi_inv);
c=AA*(thet*c(-1)+BB*c(+1)-bet*thet*c(+2)-((1-thet)/sig)*((1-bet*thet)*(r-pie(+1))-eps_b+(1+bet*thet)*eps_b(+1)-bet*thet*eps_b(+2)));
q=-(r-pie(+1))+bet*((1-delt)*q(+1)+R_bar_k*r_k(+1))+eta_q;
inv=(1/(1+bet))*(inv(-1)+bet*inv(+1)+zet*q+bet*(eps_i(+1)-eps_i));
k=(1-delt)*k(-1)+delt*inv;
u=psi*r_k;
l=-w+r_k+u+k(-1);
y=c_y*c+g_y*eps_g+delt*k_y*inv+R_bar_k*k_y*u;
y=phi*(eps_a+alph*(u+k(-1))+(1-alph)*l);
w=(1/(1+bet))*(bet*w(+1)+w(-1)+bet*pie(+1)-(1+bet*gam_w)*pie+gam_w*pie(-1)-DD*(w-chi*l-eps_l+(bet*thet/(1-bet*thet))*(eps_b-eps_b(+1))-eta_w-EE*((1+bet*thet^2)*c-thet*c(-1)-bet*thet*c(+1))));
pie=(1/(1+bet*gam_p))*(bet*pie(+1)+gam_p*pie(-1)+FF*(alph*r_k+(1-alph)*w-eps_a+eta_pie));
eps_b=rho_b*eps_b(-1)+nu_b; 
eps_i=rho_i*eps_i(-1)+nu_i; 
eps_g=rho_g*eps_g(-1)+nu_g;
eps_a=rho_a*eps_a(-1)+nu_a;
eps_l=rho_l*eps_l(-1)+nu_l;
r=r_i*r(-1)+(1-r_i)*(r_pi*(pie(-1)-pi_bar(-1))+r_y*(y(-1)-y_star(-1)))+r_delta_pi*(pie-pie(-1))+r_delta_y*(y-y_star-(y(-1)-y_star(-1)))+eta_r;
pi_bar=rho_pi_bar*pi_bar(-1)+nu_pi_bar;
c_star=AA*(thet*c_star(-1)+BB*c_star(+1)-bet*thet*c_star(+2)-((1-thet)/sig)*((1-bet*thet)*(r_star)-eps_b+(1+bet*thet)*eps_b(+1)-bet*thet*eps_b(+2)));
q_star=-r_star+bet*(1-delt)*q_star(+1)+ bet*R_bar_k*r_k_star(+1);
inv_star=(1/(1+bet))*(inv_star(-1)+bet*inv_star(+1)+zet*q_star+bet*(eps_i(+1)-eps_i));
k_star=(1-delt)*k_star(-1)+delt*inv_star;
u_star=psi*r_k_star;
l_star=-w_star+r_k_star+u_star+k_star(-1);
y_star=c_y*c_star+g_y*eps_g+delt*k_y*inv_star+R_bar_k*k_y*u_star;
y_star=phi*(eps_a+alph*(u_star+k_star(-1))+(1-alph)*l_star);
w_star=(1/(1+bet))*(bet*w_star(+1)+w_star(-1)-DD*(w_star-chi*l_star-eps_l+(bet*thet/(1-bet*thet))*(eps_b-eps_b(+1))-EE*((1+bet*thet^2)*c_star-thet*c_star(-1)-bet*thet*c_star(+1))));
FF*(alph*r_k_star+(1-alph)*w_star-eps_a)=0;
end;

%----------------------------------------------------------------
% 4. Computation
%----------------------------------------------------------------

initval;
c=0;
r=0;
pie=0;
q=0;
inv=0;
k=0;
u=0;
r_k=0;
l=0;
w=0;
y=0;
eps_b=0;
eps_i=0;
eps_g=0;
eps_a=0;
eps_l=0;
pi_bar=0;
c_star=0;
inv_star=0;
y_star=0;
w_star=0;
l_star=0;
q_star=0;
r_star=0;
r_k_star=0;
u_star=0;
k_star=0;
end;

shocks;
var eta_w; stderr sig_w;
var eta_pie; stderr sig_pie;
var eta_q; stderr sig_q;
var eta_r; stderr sig_r;
var nu_b; stderr sig_b;
var nu_i; stderr sig_i;
var nu_g; stderr sig_g;
var nu_a; stderr sig_a;
var nu_l; stderr sig_l;
var nu_pi_bar; stderr sig_pie_bar;
end;

check;
steady;
stoch_simul(irf=0);

// name, initial value, lb, ub, shape, mean, stderror
estimated_params;
zet, normal_pdf, 0.5, 0.2;   
log_psi_inv,normal_pdf, 1.8593, 1.6094;   
phi_1, gamma_pdf, 0.075, 0.0125;   
chi, normal_pdf, 1.2, 0.5;   
xi_p, beta_pdf, 0.375, 0.1;   
xi_w, beta_pdf, 0.375, 0.1;  
gam_w, beta_pdf, 0.5, 0.25;   
gam_p, beta_pdf, 0.5, 0.25;   
thet,  beta_pdf, 0.7, 0.15;   
sig,  normal_pdf, 2, 0.5;    
rho_b, beta_pdf, 0.5, 0.25;  
rho_i,  beta_pdf, 0.5, 0.25;   
rho_g, beta_pdf, 0.5, 0.25;   
rho_a, beta_pdf, 0.5, 0.25;   
rho_l, beta_pdf, 0.5, 0.25;  
rho_pi_bar, beta_pdf, 0.85, 0.1;  
stderr nu_b, gamma_pdf, 0.3, 0.3;   
stderr nu_i,  gamma_pdf, 0.1, 0.1;   
stderr nu_g, gamma_pdf, 0.3, 0.3;  
stderr nu_a, gamma_pdf, 0.6, 0.6;   
stderr nu_l, gamma_pdf, 3, 3;       
stderr nu_pi_bar,  gamma_pdf, 0.1, 0.1;    
stderr eta_q, gamma_pdf, 5, 5;  
stderr eta_pie,gamma_pdf, 0.2, 0.2;    
stderr eta_r,  gamma_pdf, 0.1, 0.1;   
stderr eta_w,  gamma_pdf, 0.2, 0.2;  
r_i, normal_pdf, 1, 0.15;  
r_pi,  normal_pdf, 2, 0.5;  
r_y, gamma_pdf, 0.25, 0.25;  
r_delta_pi,  normal_pdf, 0.2, 0.1;   
r_delta_y, gamma_pdf, 0.25, 0.25;    
end;

%%estimated_params_init;
%%zet, 0.20241;
%%log_psi_inv,2.42;
%%phi_1,0.1071;
%%chi,0.75265;
%%xi_p,0.7974;
%%xi_w,0.2705;
%%gam_w,0.4492;
%%gam_p,0.1064;
%%thet,0.3698;
%%sig,2.1673;
%%rho_b,0.9054;
%%rho_i,0.2516;
%%rho_g,0.9729;
%%rho_a,0.9462;
%%rho_l,0.9815;
%%rho_pi_bar,0.99;
%%stderr nu_b,2.5144;
%%stderr nu_i,4.4334;
%%stderr nu_g,2.00;
%%stderr nu_a,0.5906;
%%stderr nu_l,2.6591;
%%stderr nu_pi_bar,0.08169;
%%stderr eta_q,3.555;
%%stderr eta_pie,4.074;
%%stderr eta_r,0.3397;
%%stderr eta_w,0.3021;
%%r_i,0.6433;
%%r_pi,2.0674;
%%r_y,0.10857;
%%r_delta_pi,0.2287;
%%r_delta_y,0.4566;
%%end;


estimated_params_bounds;
zet,0.0001,10;
log_psi_inv,-6,6;
phi_1,0.0001,10;
chi,0.0001,10;
xi_p,0.0001,0.999;
xi_w,0.0001,0.999;
gam_w,0.0001,0.999;
gam_p,0.0001,0.999;
thet,0.0001,0.999;
sig,0.0001,10;
rho_b,0.0001,0.999;
rho_i,0.0001,0.999;
rho_g,0.0001,0.999;
rho_a,0.0001,0.999;
rho_l,0.0001,0.999;
rho_pi_bar,0.0001,0.999;
stderr nu_b,0.0001,10;
stderr nu_i,0.0001,10;
stderr nu_g,0.0001,10;
stderr nu_a,0.0001,10;
stderr nu_l,0.0001,10;
stderr nu_pi_bar,0.0001,10;
stderr eta_q,0.0001,10;
stderr eta_pie,0.0001,10;
stderr eta_r,0.0001,10;
stderr eta_w,0.0001,10;
r_i,0.0001,2;
r_pi,1.0001,10;
r_y,0.0001,10;
r_delta_pi,0.0001,0.999;
r_delta_y,0.0001,0.999;
end;

varobs c r pie inv l w y;

// computes only the posterior mode for demonstration. 
//For full Metropolis simulation set mh_replic=20000. It will take several hours
estimation(datafile=us_dat,mode_file=LOWW_benchmark_5_estimate_mode,mode_compute=0,nobs=188,presample=12,mh_replic=500000,mh_nblocks=4,mh_jscale=0.18,mh_init_scale=0.01);

%%
